{{  David Ratcliff   'ratronic'   3-4-10  }}
con
   _clkmode = xtal1 + pll16x            'system clock 
   _xinfreq = 5_000_000
   #1, i_Ping, i_ReadData, i_WriteData, i_RegWrite, i_Action, i_Reset, #131, i_SyncWrite  
   dyn = 0                              'i/o pin for dynamixel servo bus
   grf = 1500                           'gripper force limit 
   gro = 560                            'gripper open limit
   grc = 1000                           'gripper close limit
   
var
  byte byteArray[50],gf,lf,wf
  word ar,bt,bs,fa,gr,ws
   
obj
  dy : "dynamixelasmbus_v0.37_04272008"
  tv : "vga_text"
  ms : "mouse"
  ad : "cd_ltc1298"
  
pub arm | i

  'initialization
  
  ad.start(1)                           'start adc                            
  tv.start(16)                          'start vga display
  tv.str(string(1,"Wait 3 seconds"))    'wait for servos to center
  ms.start(24,25)                       'start mouse
  dy.start(dyn,1_000_000)               'start dynamixel bus
  ms.bound_scales(1,1,1)                'set mouse bounding scales  
  ms.bound_limits(0,0,0,1023,1023,1023) 'set mouse bound min max limits for x, y, and scrollwheel(z)  
  ms.bound_preset(512,512,512)          'set mouse preset start values
  ws := 512                             'wrist     
  gr := gro                             'gripper   
  word[@bytearray[0]] := 150            'set moving speed all servos
  repeat i from 1 to 5
    writedata(i,32,2)
  word[@bytearray[0]] := 512            'center all servos  
  repeat i from 1 to 5                                      
    writedata(i,30,2)                                       
  bytearray[0] := 4                     'disable overload shut down for gripper 
  writedata(5,18,1)                               
                                        
  'main loop
  
  repeat                                
    bt := ms.buttons                    'get mouse button 
    bs := ms.bound_x                    'get mouse x (base position)
    ar := ms.bound_y                    'get mouse y (arm position)
    fa := 512 + ms.abs_z * 5            'get mouse scroll wheel (forearm position)
    if lf == 0                          'level flag
      fa := fa - ms.abs_y               'keep forearm level with upper arm movement   
    if bt == 1                          'if mouse left button - open / close gripper 
      gripper
      !gf                               'toggle gripper direction flag
    if bt == 2                          'if mouse right button - move wrist cw / ccw 
      wrist
      !wf
    if bt == 4                          'if scroll wheel button pressed toggle level flag
      level
      
  'set min/max limits
            
    ar := 120 #> ar <# 620              'arm 
    bs := 390 #> bs <# 1023             'base        
    fa := 120 #> fa <# 910              'forearm
    gr := gro #> gr <# grc              'gripper
    ws := 0   #> ws <# 1023             'wrist
       
  'write position values to servos
            
    word[@bytearray[0]] := bs           'base goal position
    writedata(1,30,2)
    word[@bytearray[0]] := ar           'arm goal position 
    writedata(2,30,2)
    word[@bytearray[0]] := fa           'forearm goal position
    writedata(3,30,2)
    word[@bytearray[0]] := ws           'wrist goal position
    writedata(4,30,2)
    word[@bytearray[0]] := gr           'gripper goal position 
    writedata(5,30,2)         
    display                             'display all values
     
pub display

  'update screen with all values
  
  tv.out(1)
  tv.str(string("Servo positions",13))                         
  tv.str(string("Base      = "))    
  tv.dec(bs)                        
  tv.str(string("     ",13))        
  tv.str(string("Arm       = "))    
  tv.dec(ar)                        
  tv.str(string("     ",13))        
  tv.str(string("Forearm   = "))    
  tv.dec(fa)                        
  tv.str(string("     ",13))        
  tv.str(string("Wrist     = "))    
  tv.dec(ws)                        
  tv.str(string("     ",13))        
  tv.str(string("Gripper   = "))    
  tv.dec(gr)                        
  tv.str(string("     ",13,13))        
  tv.str(string("Gripper load = " ))   
  tv.dec(4095 - ad.getadc(1))              
  tv.str(string("     ",13))        

pub gripper

  'toggle gripper open or closed
  
  repeat
    if gf == 0                          'close gripper
      if gr == grc                      'if gripper closed then stop
        return
      if 4095 - ad.getadc(1) > grf      'if force sensor at limit then stop
        return                            
      gr++                              'increase gripper position   
      if gr == grc                      'if gripper closed then stop
        return                            
      word[@bytearray[0]] := gr         'move gripper servo
      writedata(5,30,2)                 
      waitcnt(1000000 + cnt)             
    if gf == 255                        'open gripper
      if gr == gro                      'if gripper open then stop
        return
      gr := gro                         'move gripper servo
      word[@bytearray[0]] := gr          
      writedata(5,30,2)
      waitcnt(80000000 + cnt)
      return                              
  
pub level

  'toggle level flag
  
  if lf == 0
    !lf
    return
  if lf == 255
    !lf
    return  
          
pub wrist

  'toggle wrist clockwise/counterclockwise
  
  repeat
    if wf == 0                          'rotate wrist counterclockwise
      if ws == 1023                     'if wrist rotated then stop
        return
      if ms.buttons == 0                'if mouse button not pressed then stop
        return                  
      word[@bytearray[0]] := ws         'move wrist servo
      writedata(4,30,2)          
      ws++                        
      if ws == 1023                     'if wrist at end then stop
        return
      waitcnt(500000 + cnt)  
    if wf == 255                        'rotate wrist clockwise
      if ws == 0                        'if wrist rotated then stop
        return
      if ms.buttons == 0                'if mouse button not pressed then stop
        return
      word[@bytearray[0]] := ws         'move wrist servo
      writedata(4,30,2)          
      ws--                        
      if ws == 0                        'if wrist at end then stop
        return
      waitcnt(500000 + cnt)                   
  

  
  
                                                      {dynabus com methods by Mike Gebhard}
                                                               
PUB ReadData(id, startAddr, bytesToRead) | len, cs, pl, mode
{{ReadData(<id>, <start address>, <number of bytes to read>)}}
  len := $04
  cs := (id + len + i_ReadData + startAddr + bytesToRead)^$FF
  dy.SetAxBuffer(0, $FF)
  dy.SetAxBuffer(1, $FF)
  dy.SetAxBuffer(2, id)
  dy.SetAxBuffer(3, len)
  dy.SetAxBuffer(4, i_ReadData)
  dy.SetAxBuffer(5, startAddr)
  dy.SetAxBuffer(6, bytesToRead)
  dy.SetAxBuffer(7, cs)
  pl := len + $04
  mode := GetResponseMode(i_ReadData)
  dy.ExecuteCommand(pl, mode)
    
PUB WriteData(id, startAddr, N) | len, pl, cs, mode, tempSum, idx, idx2
{{WriteData(<id>, <start address>, <number of bytes to write>)}} 
  len := N + 3
  idx := 0
  idx2 := 0
  tempSum := 0
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, id)
  dy.SetAxBuffer(idx++, len)
  dy.SetAxBuffer(idx++, i_WriteData)
  dy.SetAxBuffer(idx++, startAddr)
  'get all data bytes to send
  repeat N
    tempSum += byteArray[idx2]
    dy.SetAxBuffer((idx), byteArray[idx2])
    idx++
    idx2++
  'CheckSum  
  cs := (id + len + i_WriteData + startAddr + tempSum)^$FF
  'Set checksum
  dy.SetAxBuffer((idx), cs)
  'Packet length and mode
  pl := len + $04
  mode := GetResponseMode(i_WriteData)
  'Execute
  dy.ExecuteCommand(pl, mode)
   
PUB RegWrite(id, startAddr, N)  | len, pl, cs, mode, tempSum, idx, idx2
{{RegWrite(<id>, <start address>, <number of bytes to write>)
  You must execute the Action command after the RegWrite command}}
  len := N + 3
  idx := 0
  idx2 := 0
  tempSum := 0
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, id)
  dy.SetAxBuffer(idx++, len)
  dy.SetAxBuffer(idx++, i_RegWrite)
  dy.SetAxBuffer(idx++, startAddr)
  'get all data bytes to send
  repeat N
    tempSum += byteArray[idx2]
    dy.SetAxBuffer((idx), byteArray[idx2])
    idx++
    idx2++
  'CheckSum  
  cs := (id + len + i_RegWrite + startAddr + tempSum)^$FF
  'Set checksum
  dy.SetAxBuffer((idx), cs)
  'Packet length and mode
  pl := len + $04
  mode := GetResponseMode(i_RegWrite)
  'Execute
  dy.ExecuteCommand(pl, mode)
  
PUB Action | mode, pl, cs, len, id
  len := $02
  id := $FE
  cs := (id + len + i_Action)^$FF
  dy.SetAxBuffer(0, $FF)
  dy.SetAxBuffer(1, $FF)
  dy.SetAxBuffer(2, id)
  dy.SetAxBuffer(3, len)
  dy.SetAxBuffer(4, i_Action)
  dy.SetAxBuffer(5, cs)
  pl := len + $04
  mode := GetResponseMode(i_Action)
  'Execute
  dy.ExecuteCommand(pl,mode)  
    
PUB SyncWrite(numDynamixels, startAddr, numBytes) | pl, cs, mode, tempSum, idx, idx2, id, len
{{SyncWrite(<length>,<start address>,<number if dynamixles>, <data lenght>)
number of dynamixels, # of bytes to write, starting address, data[]     }}
  idx := 0
  idx2 := 0
  tempSum := 0
  id := $FE
  len := ((numBytes + 1) * numDynamixels)
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, $FF)
  dy.SetAxBuffer(idx++, id)
  'Half of the length equations was done above 
  dy.SetAxBuffer(idx++, len + 4)
  dy.SetAxBuffer(idx++, i_SyncWrite)
  dy.SetAxBuffer(idx++, startAddr)
  dy.SetAxBuffer(idx++, numBytes)
  'get all data bytes to send
  repeat len
    tempSum += byteArray[idx2]
    dy.SetAxBuffer((idx++), byteArray[idx2++])
  'CheckSum  
  cs := (id + (len + 4) + numBytes + i_SyncWrite + startAddr + tempSum)^$FF
  'Set checksum
  dy.SetAxBuffer((idx), cs)
  'Packet length and mode
  pl := len + 4 + 4
  mode := GetResponseMode(i_RegWrite)
  'Execute
  dy.ExecuteCommand(pl, mode)
 
PRI GetResponseMode(instructionType) : mode
  case instructionType
    I_Ping          : mode := 1
    I_ReadData      : mode := 1
    I_WriteData     : mode := 1
    I_RegWrite      : mode := 2
    I_Action        : mode := 2
    I_Reset         : mode := 1
    I_SyncWrite     : mode := 2
    other           : mode := 0
                                                                                                          